/// <reference types="node" />
import http from 'http';
import { Stream } from 'stream';
import { URL } from 'url';
export declare type HttpMethod = 'GET' | 'HEAD' | 'POST' | 'PUT' | 'DELETE' | 'OPTIONS' | 'PATCH';
export interface ServerRequest<Params = unknown, RequestBody = unknown> extends Omit<http.IncomingMessage, 'url'> {
    body: RequestBody;
    params: Params;
    protocol: 'http' | 'https';
    url: URL;
}
export interface ServerResponse extends http.ServerResponse {
}
export declare type ResponseBodyType = string | object | number | Buffer | Stream | null;
export declare type Next = (req: ServerRequest, res: ServerResponse) => Promise<void>;
export declare type Handler<ResponseBody extends ResponseBodyType = ResponseBodyType, Request extends ServerRequest = ServerRequest> = (req: Request, res: ServerResponse, next: Next) => ResponseBody | Promise<ResponseBody>;
export interface ServeOptions {
    trustProxy?: boolean;
    onError?: (err: Error) => void | Promise<void>;
}
export declare function serve(handler: Handler, options?: ServeOptions): (req: http.IncomingMessage, res: http.ServerResponse) => Promise<void>;
export declare function getHeader(req: ServerRequest, header: string): string | undefined;
export interface RequestBodyOptions {
    limit?: string;
    encoding?: string;
}
export declare function buffer(req: ServerRequest, { limit, encoding }?: RequestBodyOptions): Promise<Buffer>;
export declare function text(req: ServerRequest, options?: RequestBodyOptions): Promise<string>;
export declare function json(req: ServerRequest, options?: RequestBodyOptions): Promise<any>;
export declare function send(res: ServerResponse, code: number, body?: ResponseBodyType): void;
export declare function sendError(res: ServerResponse, error: HttpError): void;
export declare function notFound(res: ServerResponse): void;
export declare function router(...middleware: Handler<ResponseBodyType, ServerRequest<any, any>>[]): (req: ServerRequest, res: ServerResponse) => Promise<void>;
export declare type RouteHandler<Method extends HttpMethod = HttpMethod, Route extends string = string, ResponseBody extends ResponseBodyType = ResponseBodyType, Request extends ServerRequest = ServerRequest> = Handler<ResponseBody, Request> & {
    method: Method;
    route: Route;
    compilePath: (params?: Request['params']) => string;
};
export declare function route<ResponseBody extends ResponseBodyType, Method extends HttpMethod = HttpMethod, Route extends string = string>(method: Method, path: Route, handler: Handler<ResponseBody, ServerRequest<RouteParams<Route>>>): RouteHandler<Method, Route, ResponseBody, ServerRequest<RouteParams<Route>>>;
export declare function route<RequestBody extends object = object, ResponseBody extends ResponseBodyType = ResponseBodyType, Method extends HttpMethod = HttpMethod, Route extends string = string>(method: Method, path: Route, handler: Handler<ResponseBody, ServerRequest<RouteParams<Route>>>, validator: (body: object) => body is RequestBody): RouteHandler<Method, Route, ResponseBody, ServerRequest<RouteParams<Route>, RequestBody>>;
export interface HttpError extends Error {
    statusCode?: number;
    originalError?: Error;
}
export declare function createError(code: number, message: string, original?: Error): HttpError;
export declare type RouteParams<T extends string> = T extends `${string}:${infer P}?/${infer Rest}` ? {
    [K in P]?: string;
} & RouteParams<Rest> : T extends `${string}:${infer P}/${infer Rest}` ? {
    [K in P]: string;
} & RouteParams<Rest> : T extends `${string}:${infer P}?` ? {
    [K in P]?: string;
} : T extends `${string}:${infer P}` ? {
    [K in P]: string;
} : {};
//# sourceMappingURL=zap.d.ts.map